package org.processmining.plugins.cheetah.ppmchart;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.plugins.cheetah.ppmchart.model.PPMChartModel;
import org.processmining.plugins.cheetah.ppmchart.ui.*;

public class PPMChartAnalysis extends JPanel implements ActionListener {

	private static final long serialVersionUID = -2720292789575028504L;

	private final PPMChartModel ccModel;

	// GUI attributes
	private final JPanel centerPanel = new JPanel();
	private final JTabbedPane tabbedPane = new JTabbedPane();
	private JSplitPane sp = new JSplitPane();
	private JSplitPane split = new JSplitPane();
	private JSplitPane spom = new JSplitPane();
	private OverviewPanel oPanel = null;
	private FilterPanel fPanel = null;
	private PPMChartPanel cheetahChartPanel = null;
	private PPMChartOptionPanel cheetahChartOptionPanel = null;
	private JScrollPane chartPane;
	private SettingPanel settingPanel;

	public PPMChartAnalysis(PluginContext context, PPMChartModel model) {
		ccModel = model;
		analyse();
	}

	public void analyse() {
		init();
	}

	private void init() {

		int height = getHeight() - this.getInsets().bottom - this.getInsets().top - 50;
		int width = getWidth() - this.getInsets().left - this.getInsets().right - 200;

		// initialize
		cheetahChartOptionPanel = new PPMChartOptionPanel(this);
		cheetahChartPanel = new PPMChartPanel(this);
		cheetahChartOptionPanel.registerGUIListener(cheetahChartPanel);
		settingPanel = new SettingPanel(this);

		// initialize settings of taskMap
		chartPane = new JScrollPane(cheetahChartPanel, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		chartPane.setAutoscrolls(true);

		// repaint after moving scrollbar
		AdjustmentListener adjListener = new AdjustmentListener() {
			public void adjustmentValueChanged(AdjustmentEvent e) {
				reDrawBoxOnOverview();
				cheetahChartPanel.updateUI();
				cheetahChartPanel.repaint();
			}
		};
		chartPane.getHorizontalScrollBar().addAdjustmentListener(adjListener);
		chartPane.getVerticalScrollBar().addAdjustmentListener(adjListener);
		chartPane.setPreferredSize(new Dimension(width - 200, height - 10));

		// initialize overview Panel;
		oPanel = new OverviewPanel(this);
		oPanel.setPreferredSize(new Dimension(200, 130));
		
		// initialize filter Panel;
		fPanel = new FilterPanel(cheetahChartPanel, ccModel.getLogReader());
		fPanel.setPreferredSize(new Dimension(200, 130));

		// initialize Panel including overview and metrics Panel
		spom = new JSplitPane(JSplitPane.VERTICAL_SPLIT, oPanel, fPanel);
		spom.setOneTouchExpandable(false);
		spom.setDividerLocation(131);
		spom.setDividerSize(3);

		// initialize main split panel
		split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, chartPane, spom);
		split.setDividerLocation(split.getSize().width - split.getInsets().right - split.getDividerSize() - 200);

		split.setOneTouchExpandable(true);
		split.setResizeWeight(1.0);
		split.setDividerSize(3);
		split.setOneTouchExpandable(true);

		// Add cheetah Panel and Setting Panel
		tabbedPane.add("Cheetah Chart", split);
		tabbedPane.add("Settings", settingPanel);
		tabbedPane.setSelectedIndex(0);
		tabbedPane.addChangeListener(new javax.swing.event.ChangeListener() {
			public void stateChanged(javax.swing.event.ChangeEvent e) {
				switch (tabbedPane.getSelectedIndex()) {
					case 0 :
						//						getOverviewPanel().setDrawBox(true);
						//						cheetahChartPanel.changeWidthSort();
						break;
					case 1 :
						settingPanel.initSettingPanel();
						break;
					default :
						break;
				}
			}
		});

		centerPanel.setLayout(new BorderLayout());
		centerPanel.add(tabbedPane, BorderLayout.CENTER);
		centerPanel.setMinimumSize(new Dimension(width, height));
		centerPanel.setAlignmentX(LEFT_ALIGNMENT);
		sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, cheetahChartOptionPanel.getOptionPanel(), centerPanel);//westPanel, centerPanel);
		sp.setDividerLocation(165);
		sp.setDividerSize(3);
		sp.setOneTouchExpandable(true);
		setLayout(new BorderLayout());
		this.add(sp, BorderLayout.CENTER);
		validate();
		this.repaint();
	}

	// public methods for attributes
	public PPMChartModel getCheetahChartModel() {
		return ccModel;
	}

	public PPMChartPanel getCheetahChartPanel() {
		return cheetahChartPanel;
	}

	public SettingPanel getSettingPanel() {
		return settingPanel;
	}

	public PPMChartOptionPanel getCheetahChartOptionPanel() {
		return cheetahChartOptionPanel;
	}
	
	public FilterPanel getFilterPanel() {
		return fPanel;
	}

	// methods for dealing with zoom
	public Dimension getViewportSize() {
		return chartPane.getViewport().getSize();
	}

	public int getVerticalPosition() {
		return chartPane.getViewport().getViewPosition().y;
	}

	public int getHorizontalPosition() {
		return chartPane.getViewport().getViewPosition().x;
	}

	public JScrollPane getScrollPane() {
		return chartPane;
	}

	public void setScrollBarPosition(Point aP) {
		chartPane.getViewport().validate();
		if (aP.x < 0) {
			aP.x = 0;
		}
		if (aP.y < 0) {
			aP.y = 0;
		}
		if ((aP.x + chartPane.getViewport().getSize().getWidth()) > cheetahChartPanel.getWidth()) {
			aP.x = (int) (cheetahChartPanel.getWidth() - chartPane.getViewport().getSize().getWidth());
		}
		if ((aP.y + chartPane.getViewport().getSize().getHeight()) > cheetahChartPanel.getHeight()) {
			aP.y = (int) (cheetahChartPanel.getHeight() - chartPane.getViewport().getSize().getHeight());
		}
		chartPane.getViewport().setViewPosition(aP);
	}

	public void actionPerformed(ActionEvent e) {
		ccModel.updateStatisticsPanel(cheetahChartOptionPanel);
		oPanel.setDrawBox(true);
		oPanel.repaint();
		oPanel.revalidate();
	}

	public void reDrawBoxOnOverview() {
		oPanel.repaint();
		oPanel.revalidate();
	}

	@Plugin(name = "Cheetah Chart Visualization", 
			parameterLabels = "Cheetah Chart", 
			returnTypes = PPMChartAnalysis.class, 
			returnLabels = "Cheetah Chart", 
			userAccessible = false, 
			help = "Visualizes the cheetah chart")
	@Visualizer(name = "Cheetah Chart Visualization")
	public static PPMChartAnalysis visualize(PluginContext context, PPMChartModel dcm) {
		return new PPMChartAnalysis(context, dcm);
	}
}
