package org.processmining.plugins.cheetah.ppmchart.model;

import java.util.*;

import org.deckfour.xes.model.*;
import org.processmining.plugins.cheetah.ppmchart.ui.PPMChartPanel;
import org.processmining.plugins.simplelogoperations.XLogFunctions;

public class DurationModel {
	static DurationModel theModel = null;
	static XLog theLog = null;
	static String sortStandard = null;
	private HashMap<String, Node> nodes;
	private HashMap<String, Edge> edges;
	HashMap<Node,ArrayList<Edge>> edgesIn;
	Node startEvent;
	
	public static DurationModel getInstance(XLog log, String sort) {
		if (theModel == null || theLog == null || !theLog.equals(log)) {
			theModel = new DurationModel(log);
			theLog = log;
		}
		sortStandard = sort;
		return theModel;			
	}
	private DurationModel(XLog log) {
		nodes = new HashMap<String, Node>();
		edges = new HashMap<String, Edge>();
		startEvent = null;
		
		createModel(log);
		calculateDistances();
	}
	private void createModel(XLog log) {
		for(XTrace trace : log) {
			for (XEvent event : trace) {
				XAttributeMap map = event.getAttributes();
				String activity = XLogFunctions.getName(event).toUpperCase();
				if (!map.containsKey("id"))
					continue;
				String id = map.get("id").toString();
				if (isNode(activity)) {
					long x = map.containsKey("x") ? XLogFunctions.getLongValue(map.get("x")) : 0;
					long y = map.containsKey("y") ? XLogFunctions.getLongValue(map.get("y")) : 0;
					if (isCreate(activity))
						if (nodes.containsKey(id)) {
							nodes.get(id).setType(getType(activity)).setX(x).setY(y).setDeleted(false);
							if (isStartEvent(activity))
								startEvent = nodes.get(id);
						}
						else {
							Node node = new Node(id, getType(activity), x, y);
							if (isStartEvent(activity))
								startEvent = node;
							nodes.put(id, node);
						}
					else if (isMove(activity))
						nodes.get(id).setX(x).setY(y);
					else if (isDelete(activity))
						nodes.get(id).setDeleted(true);
				}
				else if (isEdge(activity)) {
					String source = map.containsKey("source_node_id") ? XLogFunctions.getStringValue(map.get("source_node_id")) : "";
					String target = map.containsKey("target_node_id") ? XLogFunctions.getStringValue(map.get("target_node_id")) : "";
					if (isCreate(activity)) {
						if (!nodes.containsKey(source))
							nodes.put(source, new Node(source, 0, 0));
						if (!nodes.containsKey(target))
							nodes.put(target, new Node(target, 0, 0));
						edges.put(id, new Edge(id, nodes.get(source), nodes.get(target)));
					}
					else if (isReconnect(activity)) {
						Edge edge = edges.get(id).setId("OLD" + id).setDeleted(true);
						edges.put("OLD" + id, edge);
						if (!nodes.containsKey(source))
							nodes.put(source, new Node(source, 0, 0));
						if (!nodes.containsKey(target))
							nodes.put(target, new Node(target, 0, 0));
						edges.put(id, new Edge(id, nodes.get(source), nodes.get(target)));
					}
					else if (isDelete(activity)) {
						edges.get(id).setDeleted(true);
					}
				}
			}
		}
		for (String node : nodes.keySet())
			System.out.print(node + ", ");
		System.out.println();
		for (Map.Entry<String,Edge> edge : edges.entrySet())
			System.out.println(edge.getValue().getSource().getId() + "->" + edge.getValue().getTarget().getId());
	}
	private void calculateDistances() {
		edgesIn = new HashMap<Node, ArrayList<Edge>>();
		for (Map.Entry<String,Node> node : nodes.entrySet())
			edgesIn.put(node.getValue(), new ArrayList<Edge>());
		for (Map.Entry<String,Edge> edge : edges.entrySet())
			edgesIn.get(edge.getValue().getTarget()).add(edge.getValue());
		if (startEvent == null) {
			for (Map.Entry<String,Node> node : nodes.entrySet())
				if (!edgesIn.containsKey(node.getValue()) || edgesIn.get(node.getValue()).size() == 0)
					startEvent = node.getValue();
		}
	}
	private double getDistance(Node node) {
		if (node.getDistance() >= 0)
			return new Double(node.getDistance());
		
		ArrayList<Edge> edges = edgesIn.get(node);		
		if (edges.size() == 0) {
			if (node.getType() != Node.TYPE.START_EVENT)
				node.setDistance(getDistance(node, startEvent));
			else
				node.setDistance(0);
			return new Double(node.getDistance());
		}
		else if (edges.size() == 1) {
			node.setDistance(getDistance(edges.get(0)));
			return new Double(node.getDistance());
		}
		
		ArrayList<Edge> existingEdges = new ArrayList<Edge>();
		for (int i=0; i<edges.size(); i++)
			if (!edges.get(i).isDeleted())
				existingEdges.add(edges.get(i));
		
		if (existingEdges.size() == 1)
			return new Double(getDistance(existingEdges.get(0)));
		else if (existingEdges.size() == 0)
			existingEdges = edges;
		
		double distance = -1;
		for (int i=0; i<existingEdges.size(); i++)
			if (getDistance(existingEdges.get(i)) > distance)
				distance = getDistance(existingEdges.get(i));
		node.setDistance(distance);
		return new Double(node.getDistance());
	}
	private ArrayList<Edge> edgesBeingProcessed = new ArrayList<Edge>();
	private double getDistance(Edge edge) {
		if (edgesBeingProcessed.contains(edge))
			return 0;
		edgesBeingProcessed.add(edge);
		Node s = edge.getSource();
		Node t = edge.getTarget();
		if (s==null || t == null)
			getDistance(s, startEvent);
		Double startDist = getDistance(s);
		double res = startDist.doubleValue() + getDistance(s, t);
		edgesBeingProcessed.remove(edge);
		return res;
	}
	private double getDistance(Node a, Node b) {
		if (a==null || b == null)
			getDistance(a, startEvent);
		return Math.sqrt(Math.pow(a.getX() - b.getX(),2) + Math.pow(a.getY() - b.getY(),2));
	}
	
	protected Node.TYPE getType(String activity) {
		if (activity.endsWith("_ACTIVITY"))
			return Node.TYPE.ACTIVITY;
		else if (activity.endsWith("_START_EVENT"))
			return Node.TYPE.START_EVENT;
		else if (activity.endsWith("_END_EVENT"))
			return Node.TYPE.END_EVENT;
		else if (activity.endsWith("_XOR"))
			return Node.TYPE.XOR;
		else if (activity.endsWith("_AND"))
			return Node.TYPE.AND;
		else
			return Node.TYPE.UNKNOWN;
	}
	protected boolean isNode(String activity) {
		return activity.endsWith("_ACTIVITY") ||
			   activity.endsWith("_START_EVENT") ||
			   activity.endsWith("_END_EVENT") ||
			   activity.endsWith("_XOR") ||
			   activity.endsWith("_AND");
	}
	protected boolean isStartEvent(String activity) {
		return activity.endsWith("_START_EVENT");
	}
	protected boolean isEdge(String activity) {
		return activity.endsWith("_EDGE");
	}
	protected boolean isCreate(String activity) {
		return activity.startsWith("CREATE_");
	}
	protected boolean isMove(String activity) {
		return activity.startsWith("MOVE_");
	}
	protected boolean isReconnect(String activity) {
		return activity.startsWith("RECONNECT_");
	}
	protected boolean isDelete(String activity) {
		return activity.startsWith("DELETE_");
	}
	
	public long getDistanceValue(String id) {
		if (nodes.containsKey(id))
				return Math.round(getDistance(nodes.get(id)));
		else if (edges.containsKey(id)) {
			double distSource = getDistance(edges.get(id).getSource());
			double distTarget = getDistance(edges.get(id).getTarget());
			if (sortStandard.equals(PPMChartPanel.ST_DIST_START))
				return Math.round((distSource + distTarget) / 2.0);
			else
				return (long) (Math.max(distSource, distTarget) + 1);
		}
		else
			return 0;
	}
}
