package org.processmining.plugins.cheetah.ppmchart.ui;

import java.awt.Color;
import java.util.*;

public class ColorReference {

	protected HashMap<String, Color> mappings = null;
	protected Color[] colors = null;
	protected boolean[] assigned = null;
	protected Random rnd = null;

	public ColorReference() {
		mappings = new HashMap<String, Color>();
		rnd = new Random();
		// standard colors
		colors = new Color[16];
		colors[0] = new Color(0, 255, 0);     // green
		colors[1] = new Color(0, 153, 0);     // dark green
		colors[2] = new Color(204, 255, 204); // light green
		colors[3] = new Color(0, 204, 204);   // blue green
		colors[4] = new Color(255, 0, 0);     // red
		colors[5] = new Color(153, 0, 0);     // dark red
		colors[6] = new Color(255, 204, 204); // light red
		colors[7] = new Color(255, 153, 0);   // orange
		colors[8] = new Color(0, 0, 255);     // blue
		colors[9] = new Color(0, 0, 153);     // dark blue
		colors[10] = new Color(204, 204, 255); // light blue
		colors[11] = new Color(153, 0, 255);   // purple
		colors[12] = new Color(255, 0, 255);   // pink
		colors[13] = new Color(153, 153, 153); // grey
		colors[14] = new Color(102, 102, 102); // dark grey
		colors[15] = new Color(204, 204, 204); // light grey
		mappings.put("CREATE_ACTIVITY", colors[0]);
		mappings.put("CREATE_AND", colors[1]);
		mappings.put("CREATE_EDGE", colors[3]);
		mappings.put("CREATE_EDGE_BENDPOINT", colors[13]);
		mappings.put("CREATE_END_EVENT", colors[2]);
		mappings.put("CREATE_START_EVENT", colors[2]);
		mappings.put("CREATE_XOR", colors[1]);
		mappings.put("DELETE_ACTIVITY", colors[4]);
		mappings.put("DELETE_AND", colors[5]);
		mappings.put("DELETE_EDGE", colors[7]);
		mappings.put("DELETE_EDGE_BENDPOINT", colors[14]);
		mappings.put("DELETE_END_EVENT", colors[6]);
		mappings.put("DELETE_START_EVENT", colors[6]);
		mappings.put("DELETE_XOR", colors[5]);
		mappings.put("MOVE_ACTIVITY", colors[8]);
		mappings.put("MOVE_AND", colors[9]);
		mappings.put("MOVE_EDGE_BENDPOINT", colors[15]);
		mappings.put("MOVE_EDGE_LABEL", colors[15]);
		mappings.put("MOVE_END_EVENT", colors[10]);
		mappings.put("MOVE_START_EVENT", colors[10]);
		mappings.put("MOVE_XOR", colors[9]);
		mappings.put("RECONNECT_EDGE", colors[11]);
		mappings.put("NAME_ACTIVITY", colors[12]);
		mappings.put("NAME_EDGE", colors[12]);
		mappings.put("RENAME_ACTIVITY", colors[12]);
		mappings.put("RENAME_EDGE", colors[12]);
		// assignment map
		assigned = new boolean[16];
		for (int i = 0; i < assigned.length; i++) {
			assigned[i] = true;
		}
	}

	public Color getColor(String key) {
		if (mappings.containsKey(key)) {
			// return previously mapped color
			return mappings.get(key);
		} else {
			// check for available standard color
			for (int i = 0; i < assigned.length; i++) {
				if (assigned[i] == false) {
					assigned[i] = true;
					mappings.put(key, colors[i]);
					return colors[i];
				}
			}
			// create random color
			Color random = new Color(rnd.nextInt(245), rnd.nextInt(245), rnd.nextInt(245));
			mappings.put(key, random);
			return random;
		}
	}

	public void freeColor(String key) {
		Color c = mappings.remove(key);
		for (int i = 0; i < colors.length; i++) {
			if (colors[i].equals(c)) {
				assigned[i] = false;
				return;
			}
		}
	}

	public void assignColor(String key, Color color) {
		mappings.put(key, color);
	}

	public void randomizeColor(String key) {
		Color random = new Color(rnd.nextInt(245), rnd.nextInt(245), rnd.nextInt(245));
		mappings.put(key, random);
	}
}
