package org.processmining.plugins.cheetah.ppmchart.ui;

import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;

import javax.swing.*;

import org.deckfour.xes.model.*;
import org.processmining.plugins.simplelogoperations.XLogFunctions;

public class FilterPanel extends JPanel implements ItemListener{

	private static final long serialVersionUID = -2720292789575028503L;
	
	protected PPMChartPanel ccPanel = null;
	
	private final JCheckBox actCheckBox = new JCheckBox("activity", false);
	private final JCheckBox staCheckBox = new JCheckBox("start event", false);
	private final JCheckBox endCheckBox = new JCheckBox("end event", false);
	private final JCheckBox andCheckBox = new JCheckBox("and gateway", false);
	private final JCheckBox xorCheckBox = new JCheckBox("xor gateway", false);
	private final JCheckBox edgCheckBox = new JCheckBox("edge", false);
	
	private final JCheckBox creaCheckBox = new JCheckBox("create", false);
	private final JCheckBox moveCheckBox = new JCheckBox("move", false);
	private final JCheckBox deleCheckBox = new JCheckBox("delete", false);
	private final JCheckBox nameCheckBox = new JCheckBox("(re)name", false);

	private ArrayList<String> actInstances;
	private ArrayList<String> staInstances;
	private ArrayList<String> endInstances;
	private ArrayList<String> andInstances;
	private ArrayList<String> xorInstances;
	private ArrayList<String> edgInstances;
	private ArrayList<String> creaInstances;
	private ArrayList<String> moveInstances;
	private ArrayList<String> deleInstances;
	private ArrayList<String> nameInstances;
	private ArrayList<String> creaTasks;
	private ArrayList<String> moveTasks;
	private ArrayList<String> deleTasks;
	private ArrayList<String> nameTasks;
	
	private final JCheckBox allItemsCheckBox = new JCheckBox("hide all actions of items", false);
	
	public FilterPanel(PPMChartPanel ccPanel, XLog log) {
		this.ccPanel = ccPanel;
		initLists(log);
		
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setAlignmentX(Component.LEFT_ALIGNMENT);
		
		add(new JLabel("Hide next items"));
		add(actCheckBox);
		add(staCheckBox);
		add(endCheckBox);
		add(andCheckBox);
		add(xorCheckBox);
		add(edgCheckBox);
		add(Box.createRigidArea(new Dimension(5, 10)));
		
		add(new JLabel("Hide next actions"));
		add(creaCheckBox);
		add(moveCheckBox);
		add(deleCheckBox);
		add(nameCheckBox);
		add(Box.createRigidArea(new Dimension(5, 10)));
		add(allItemsCheckBox);
		add(new JLabel("       with these actions"));
				
		actCheckBox.addItemListener(this);
		staCheckBox.addItemListener(this);
		endCheckBox.addItemListener(this);
		andCheckBox.addItemListener(this);
		xorCheckBox.addItemListener(this);
		edgCheckBox.addItemListener(this);
		
		creaCheckBox.addItemListener(this);
		moveCheckBox.addItemListener(this);
		deleCheckBox.addItemListener(this);
		nameCheckBox.addItemListener(this);
		allItemsCheckBox.addItemListener(this);
	}
	protected void initLists(XLog log) {
		actInstances = new ArrayList<String>();
		staInstances = new ArrayList<String>();
		endInstances = new ArrayList<String>();
		andInstances = new ArrayList<String>();
		xorInstances = new ArrayList<String>();
		edgInstances = new ArrayList<String>();
		creaInstances = new ArrayList<String>();
		moveInstances = new ArrayList<String>();
		deleInstances = new ArrayList<String>();
		nameInstances = new ArrayList<String>();
		creaTasks = new ArrayList<String>();
		moveTasks = new ArrayList<String>();
		deleTasks = new ArrayList<String>();
		nameTasks = new ArrayList<String>();
		
		for (XTrace trace : log) {
			String id = XLogFunctions.getName(trace);
			for (XEvent event : trace) {
				String activity = XLogFunctions.getName(event);
				if (activity.endsWith("_BENDPOINT")) {
					moveTasks.add(activity);
					if (!moveInstances.contains(id)) moveInstances.add(id);
					if (!edgInstances.contains(id)) edgInstances.add(id);
				}
				else if (activity.equals("RECONNECT_EDGE")) {
					creaTasks.add(activity);
					deleTasks.add(activity);
					if (!creaInstances.contains(id)) creaInstances.add(id);
					if (!deleInstances.contains(id)) deleInstances.add(id);
				}
				else if (activity.startsWith("CREATE_")) {
					creaTasks.add(activity);
					if (!creaInstances.contains(id)) creaInstances.add(id);
				}
				else if (activity.startsWith("MOVE_")) {
					moveTasks.add(activity);
					if (!moveInstances.contains(id)) moveInstances.add(id);
				}
				else if (activity.startsWith("DELETE_")) {
					deleTasks.add(activity);
					if (!deleInstances.contains(id)) deleInstances.add(id);
				}
				else if (activity.contains("NAME_")) {
					nameTasks.add(activity);
					if (!nameInstances.contains(id)) nameInstances.add(id);
				}
				else {
					activity = "" + activity;
				}
				
				if (activity.endsWith("_ACTIVITY")) {
					if (!actInstances.contains(id)) actInstances.add(id);
				}
				else if (activity.endsWith("_START_EVENT")) {
					if (!staInstances.contains(id)) staInstances.add(id);
				}
				else if (activity.endsWith("_END_EVENT")) {
					if (!endInstances.contains(id)) endInstances.add(id);
				}
				else if (activity.endsWith("_AND")) {
					if (!andInstances.contains(id)) andInstances.add(id);
				}
				else if (activity.endsWith("_XOR")) {
					if (!xorInstances.contains(id)) xorInstances.add(id);
				}
				else if (activity.endsWith("_EDGE") || activity.equals("MOVE_EDGE_LABEL")) {
					if (!edgInstances.contains(id)) edgInstances.add(id);
				}
			}
		}
	}

	public void itemStateChanged(ItemEvent e) {		
		ccPanel.initBufferedImage();
		ccPanel.revalidate();
		ccPanel.repaint();		
	}
	public boolean hideTask(String task) {
		if (creaCheckBox.isSelected() && creaTasks.contains(task)) return true;
		if (moveCheckBox.isSelected() && moveTasks.contains(task)) return true;
		if (deleCheckBox.isSelected() && deleTasks.contains(task)) return true;
		if (nameCheckBox.isSelected() && nameTasks.contains(task)) return true;

		return false;
	}
	public boolean hideInstance(String instance) {
		if (actCheckBox.isSelected() && actInstances.contains(instance)) return true;
		if (staCheckBox.isSelected() && staInstances.contains(instance)) return true;
		if (endCheckBox.isSelected() && endInstances.contains(instance)) return true;
		if (andCheckBox.isSelected() && andInstances.contains(instance)) return true;
		if (xorCheckBox.isSelected() && xorInstances.contains(instance)) return true;
		if (edgCheckBox.isSelected() && edgInstances.contains(instance)) return true;

		if (allItemsCheckBox.isSelected()) {
			if (creaCheckBox.isSelected() && creaInstances.contains(instance)) return true;
			if (moveCheckBox.isSelected() && moveInstances.contains(instance)) return true;
			if (deleCheckBox.isSelected() && deleInstances.contains(instance)) return true;
			if (nameCheckBox.isSelected() && nameInstances.contains(instance)) return true;
		}
		
		return false;
	}
}
