package org.processmining.plugins.cheetah.ppmchart.ui;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import org.processmining.plugins.cheetah.ppmchart.PPMChartAnalysis;
import org.processmining.plugins.cheetah.ppmchart.model.PPMChartModel;

public class SettingPanel extends JPanel {

	private static final long serialVersionUID = -2720292789575028505L;

	protected JLabel colorsettingLabel = new JLabel("Change colors by pressing buttons");
	protected ColorReference colorReference;
	private JPanel colorPanel;
	private JScrollPane colorScrollPane;
	private JPanel colorMainPanel;

	private final PPMChartAnalysis cheetahChartAnalysis;
	private PPMChartPanel cheetahChartPanel;
	private PPMChartModel ccModel;

	// for color reference
	private JLabel[] tempLable;
	private JButton[] tempButton;
	private JPanel colorChoosePanel;
	private JPanel colorMPanel;

	//	private JTextField colorFile = new JTextField();
	//	private JButton chooseColorButton = new JButton();
	private JLabel fbackColorLabel;
	private JLabel sbackColorLabel;
	private JButton fbackColorButton = new JButton();
	private JButton sbackColorButton = new JButton();

	private Color colorLogDark = new Color(170, 170, 160);
	private Color colorLogBright = new Color(210, 210, 200);

	public SettingPanel(PPMChartAnalysis aCheetahChartAnalysis) {
		cheetahChartAnalysis = aCheetahChartAnalysis;
		cheetahChartPanel = aCheetahChartAnalysis.getCheetahChartPanel();
		ccModel = aCheetahChartAnalysis.getCheetahChartModel();
	}

	public void initSettingPanel() {
		if (cheetahChartPanel == null) {
			cheetahChartPanel = cheetahChartAnalysis.getCheetahChartPanel();
			ccModel = cheetahChartAnalysis.getCheetahChartModel();
		}
		removeAll();
		// event panel
		JPanel p0 = new JPanel();
		p0.setLayout(new BoxLayout(p0, BoxLayout.PAGE_AXIS));
		p0.add(Box.createRigidArea(new Dimension(5, 10)));

		// init paint panel
		initColorPanel();
	}

	// method dealing with color
	private void initColorPanel() {

		colorReference = cheetahChartPanel.getColorReference();

		String type = cheetahChartAnalysis.getCheetahChartOptionPanel().getColorStandard();

		if (type.equals(PPMChartModel.STR_NONE)) {
			return;
		}
		ArrayList<String> keySet = ccModel.getItemArrayList(type);

		colorPanel = new JPanel(new GridLayout(keySet.size(), 1));
		colorPanel.setPreferredSize(new Dimension(250, keySet.size() * 15));
		colorPanel.setMaximumSize(new Dimension(250, keySet.size() * 15));

		tempLable = new JLabel[keySet.size()];
		tempButton = new JButton[keySet.size()];

		int i = 0;
		for (Iterator<String> itr = keySet.iterator(); itr.hasNext();) {
			String tempString = itr.next();
			tempLable[i] = new JLabel(tempString + ":");
			tempLable[i].setToolTipText(tempString);
			tempButton[i] = new JButton("push to change");
			tempButton[i].setForeground(colorReference.getColor(tempString));
			tempButton[i].setToolTipText(tempString);
			tempButton[i].setActionCommand(tempString);
			tempButton[i].addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					JButton tempButton = (JButton) e.getSource();
					Color newColor = JColorChooser.showDialog(tempButton, "Choose Background Color", tempButton
							.getBackground());
					if (newColor != null) {
						tempButton.setForeground(newColor);
						assignColor(tempButton.getActionCommand(), newColor);
					}
				}
			});
			colorChoosePanel = new JPanel(new GridLayout(1, 2));
			colorChoosePanel.add(tempLable[i]);
			colorChoosePanel.add(tempButton[i]);
			colorPanel.add(colorChoosePanel);
			i++;
		}
		colorMPanel = null;
		colorMPanel = new JPanel();
		colorMPanel.setLayout(new BoxLayout(colorMPanel, BoxLayout.Y_AXIS));
		colorMPanel.add(colorPanel);
		colorScrollPane = new JScrollPane(colorMPanel);
		colorScrollPane.setPreferredSize(new Dimension(260, 410));
		colorMainPanel = new JPanel();
		colorMainPanel.setLayout(new BoxLayout(colorMainPanel, BoxLayout.Y_AXIS));
		JLabel changeColorLabel = new JLabel("           Set colors");
		colorMainPanel.add(changeColorLabel);
		colorMainPanel.add(colorScrollPane);

		sbackColorLabel = new JLabel("First Background");
		sbackColorLabel.setToolTipText("First Background");
		sbackColorButton = new JButton("push to change");
		sbackColorButton.setForeground(colorLogBright);
		sbackColorButton.setToolTipText("First Background");
		sbackColorButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JButton tempButton = (JButton) e.getSource();
				Color newColor = JColorChooser.showDialog(tempButton, "Choose Background Color", tempButton
						.getBackground());
				if (newColor != null) {
					tempButton.setForeground(newColor);
					colorLogBright = newColor;
				}
			}
		});
		fbackColorLabel = new JLabel("Second Background");
		fbackColorLabel.setToolTipText("Second Background");
		fbackColorButton = new JButton("push to change");
		fbackColorButton.setForeground(colorLogDark);
		fbackColorButton.setToolTipText("Second Background");
		fbackColorButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JButton tempButton = (JButton) e.getSource();
				Color newColor = JColorChooser.showDialog(tempButton, "Choose Background Color", tempButton
						.getBackground());
				if (newColor != null) {
					tempButton.setForeground(newColor);
					colorLogDark = newColor;
				}
			}
		});
		JPanel tempPanel = new JPanel(new GridLayout(1, 2));
		tempPanel.add(sbackColorLabel);
		tempPanel.add(sbackColorButton);
		tempPanel.add(fbackColorLabel);
		tempPanel.add(fbackColorButton);
		colorMainPanel.add(tempPanel);

		this.add(colorMainPanel);
	}

	private void redrawColorPart() {
		String type = cheetahChartAnalysis.getCheetahChartOptionPanel().getColorStandard();
		colorMPanel.remove(colorPanel);
		this.remove(colorMainPanel);

		if (type.equals(PPMChartModel.STR_NONE)) {
			return;
		}
		ArrayList<String> keySet = ccModel.getItemArrayList(type);

		colorPanel = null;
		colorPanel = new JPanel(new GridLayout(keySet.size(), 1));
		colorPanel.setPreferredSize(new Dimension(250, keySet.size() * 15));
		colorPanel.setMaximumSize(new Dimension(250, keySet.size() * 15));

		int i = 0;
		for (Iterator<String> itr = keySet.iterator(); itr.hasNext();) {
			String tempString = itr.next();
			tempLable[i] = new JLabel(tempString + ":");
			tempLable[i].setToolTipText(tempString);
			tempButton[i] = new JButton("push to change");
			tempButton[i].setForeground(colorReference.getColor(tempString));
			tempButton[i].setToolTipText(tempString);
			tempButton[i].setActionCommand(tempString);
			tempButton[i].addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					JButton tempButton = (JButton) e.getSource();
					Color newColor = JColorChooser.showDialog(tempButton, "Choose Background Color", tempButton
							.getBackground());
					if (newColor != null) {
						tempButton.setBackground(newColor);
						assignColor(tempButton.getActionCommand(), newColor);
					}
				}
			});
			colorChoosePanel = new JPanel(new GridLayout(1, 2));
			colorChoosePanel.add(tempLable[i]);
			colorChoosePanel.add(tempButton[i]);
			colorPanel.add(colorChoosePanel);
			i++;
		}
		colorMPanel.add(colorPanel);
		colorMPanel = null;
		colorMPanel = new JPanel();
		colorMPanel.setLayout(new BoxLayout(colorMPanel, BoxLayout.Y_AXIS));
		colorMPanel.add(colorPanel);
		colorMPanel.repaint();
		colorMPanel.revalidate();
		colorScrollPane = new JScrollPane(colorMPanel);
		colorScrollPane.setPreferredSize(new Dimension(260, 410));
		colorMainPanel = null;
		colorMainPanel = new JPanel();
		colorMainPanel.setLayout(new BoxLayout(colorMainPanel, BoxLayout.Y_AXIS));
		JLabel changeColorLabel = new JLabel("           Set colors");
		colorMainPanel.add(changeColorLabel);
		colorMainPanel.add(colorScrollPane);

		JPanel tempPanel = new JPanel(new GridLayout(1, 2));
		tempPanel.add(sbackColorLabel);
		tempPanel.add(sbackColorButton);
		tempPanel.add(fbackColorLabel);
		tempPanel.add(fbackColorButton);
		colorMainPanel.add(tempPanel);

		//		colorMainPanel.add(colorFile);
		//		colorMainPanel.add(chooseColorButton);

		this.add(colorMainPanel);
		this.repaint();
		revalidate();
	}


	public Color getFBcolor() {
		return colorLogDark;
	}

	public Color getSBcolor() {
		return colorLogBright;
	}

	public void assignColor(String name, Color newColor) {
		colorReference.assignColor(name, newColor);
	}

	public void changeColorPanel() {
		if (colorMainPanel != null) {
			this.remove(colorMainPanel);
		}
		colorMainPanel = null;
		initColorPanel();
		this.repaint();
	}
}
